<?php
require_once ('op5/license/op5lic.php');
require_once ('op5/sysinfo.php');
require_once ('op5/mayi.php');

class op5license_Constraints implements op5MayI_Constraints {
	protected $license_backend = false;

	public function __construct() {
		$this->license_backend = new op5license();
	}
	/**
	 * Execute a action
	 *
	 * @param string $action
	 *        	name of the action, as "path.to.resource:action"
	 * @param array $env
	 *        	environment variables for the constraints
	 * @param array $messages
	 *        	referenced array to add messages to
	 * @param array $perfdata
	 *        	referenced array to add performance data to
	 */
	public function run($action, $env, &$messages, &$perfdata) {
		list($resource, $method) = array_map(function($str) { return explode('.', $str); }, explode(':', $action) );

		if($resource[0] == 'ninja')
			return true; /* ninja resource namespace is meta-methods in ninja, which shouldn't be controlled by license */

		$result = $this->license_backend->run($action, $env, $messages, $perfdata);
		return $result;
	}
}

class op5license_Environment implements op5MayI_Actor {
	public function getActorInfo() {
		return array(
			'time' => time()
		);
	}
}

class ninja_license {
	const THRESHOLD = 0.95;



	/**
	 * Holds the instance of the op5lic class for this session
	 *
	 * @var op5lic
	 */
	private $lic = false;
	/**
	 * Interface to get usage statistics
	 *
	 * @var op5sysinfo
	 */
	private $sysinfo = false;

	/**
	 * Create hooks and initialize libraries
	 */
	public function __construct() {
		/* Skip those events, we only want to use op5v3-licenses. Those should go back soon.
		Event::add('system.post_controller_constructor',
			array ($this,'check_license'));
		Event::add('ninja.show_login', array ($this,'show_login'));
		Event::add('system.pre_controller', array ($this,'cleanup_session'));
		*/

		Event::add('ninja.get_branding', array ($this,'get_branding'));

		$this->sysinfo = op5sysinfo::instance();
		$this->lic = new op5lic();
		$mayi = op5MayI::instance();
		$mayi->be('env',new op5license_Environment());
		$mayi->act_upon(new op5license_Constraints());
	}

	/**
	 * Hook to retrieve licensed branding
	 */
	public function get_branding () {

		$license = new op5license();
		$metadata = $license->get_metadata();
		$data = &Event::$data['branding'];

		if (!empty($metadata) && isset($metadata['branding'])) {

			$data['label'] = $metadata['branding'];
			$format = 'brands/%s.png';

			try {
				$data['image'] = ninja::add_path(sprintf($format, $data['label']), false, false);
				$data['label'] = '';
			} catch (FileLookupErrorException $e) {
				// If there is no image, use default image already set
				// and the message as a label for that image.
				$data['image'] = ninja::add_path('brands/op5.png', false, true);
			}

		}

	}

	/**
	 * Hook to add some global notifications if license needs to
	 */
	public function check_license() {
		$controller = Event::$data;

		if ($controller instanceof Ninja_Controller) {

			foreach ($this->lic->fetch_messages('load') as $msg) {
				$controller->add_global_notification($msg);
			}

			/**
			*	Add notification for SLA-reports, Availability-reports,
			*	Alert History and Alert Summary
			*/
			foreach ($this->lic->fetch_messages('report') as $msg) {
				$controller->add_print_notification($msg);
			}

			try {
				/* Fetch if exceeding threshold, and in that case, the text */
				$monitor_usage = $this->sysinfo->get_monitor_usage();
				$exceeding = !$this->lic->query_threshold('monitor',
					$monitor_usage, self::THRESHOLD, $monitor_messages);

				/*
				 * Only show exceeding message if enable_extra_license_info, or
				 * actually exceeding the license
				 */
				if (!$this->lic->fetch_property('enable_extra_license_info')) {
					$exceeding = !$this->lic->query(
						array ('monitor' => $monitor_usage,'time_expire'=>false));
				}

				/* If exceeding, display messages */
				if ($exceeding) {
					foreach ($monitor_messages as $msg) {
						$controller->add_global_notification($msg);
					}
				}
			} catch (op5LivestatusException $e) {
				/*
				 * Don't bubble this exception further. Just add a notice bar.
				 * Bubble this further means exposing the license library code
				 */
				$controller->add_global_notification($e->getMessage());
			}
		}
	}

	/**
	 * Hook to replace login screen if system is unlicensed
	 */
	public function show_login() {
		$controller = Event::$data;
		if ($this->lic->fetch_property('unlicensed')) {
			$controller->template = $controller->add_view(
				'license_call_me_maybe');
		}
	}

	/**
	 * Destroy session if unlicensed.
	 * That means, we shouldn't be able to be logged in at all to ninja
	 */
	public function cleanup_session() {
		if ($this->lic->fetch_property('unlicensed')) {
			Session::destroy();

			Auth::disable_auth_subsystem();
		}
	}
}

new ninja_license();